@@ -47,12 +47,21 @@ from .core import * # pylint: disable=W0401, W0614
 ########################################################
 
 class Cloud(object):
-    def __init__(self, apiRegion=None, apiKey=None, apiSecret=None, apiDeviceID=None, new_sign_algorithm=True, initial_token=None, **extrakw):
+    def __init__(self, apiRegion=None, apiKey=None, apiSecret=None, apiDeviceID=None, new_sign_algorithm=True, initial_token=None, configFile=CONFIGFILE, **extrakw):
         """
         Tuya Cloud IoT Platform Access
 
         Args:
-            initial_token: The auth token from a previous run.  It will be refreshed if it has expired
+            apiRegion (str, optional): Tuya API region code (e.g., 'us', 'eu', 'cn', 'in').
+            apiKey (str, optional): Tuya Cloud API key.
+            apiSecret (str, optional): Tuya Cloud API secret.
+            apiDeviceID (str, optional): Device ID for initial API calls.
+            new_sign_algorithm (bool, optional): Use new sign algorithm for API requests. Default: True.
+            initial_token (str, optional): The auth token from a previous run. It will be refreshed if expired.
+            configFile (str, optional): Path to the config file to use for credentials. Default: 'tinytuya.json'.
+            **extrakw: Additional keyword arguments for future compatibility.
+
+        If apiKey or apiSecret are not provided, credentials will be loaded from the config file.
 
         Playload Construction - Header Data:
             Parameter 	  Type    Required	Description
@@ -77,7 +86,7 @@ class Cloud(object):
             * https://iot.tuya.com/cloud/products/detail
         """
         # Class Variables
-        self.CONFIGFILE = 'tinytuya.json'
+        self.CONFIGFILE = configFile
         self.apiRegion = apiRegion
         self.apiKey = apiKey
         self.apiSecret = apiSecret
@@ -894,3 +903,4 @@ class Cloud(object):
                 self.getmapping( productid, devid )
 
         return self.mappings
+
